@@ -7,18 +7,20 @@ use diagnostics;
 require Exporter;
 our @ISA = qw(Exporter);
 our @EXPORT = qw( auction );
-our $VERSION = '0.22';
+our $VERSION = '0.23';
 
 #Variables global to the package	
 my $maximize_total_benefit;
 my $matrix_spaces;     # used to print messages on the screen
 my $decimals;          # the number of digits after the decimal point
 my $max_matrix_value;
+my $need_transpose = 0;
 my $inicial_price;
 my $iter_count = 0;
-my ( $size_array1, $size_array2, $min_size, $max_size );
+my ( $array1_size, $array2_size, $min_size, $max_size, $original_max_size );
 my ( @assignment, @prices );
 my ( %assignned_object, %assignned_person, %price_object );
+my %index_correlation;
 
 sub auction { #                        => default values
    my %args = ( matrix_ref             => undef,     # reference to array: matrix N x M			                                     
@@ -31,11 +33,12 @@ sub auction { #                        => default values
        
    my @matrix_input = @{$args{matrix_ref}};          # Input: Reference to the input matrix (NxM)  
    
-   $size_array1 = $#matrix_input + 1;
-   $size_array2 = $#{$matrix_input[0]} + 1;
+   $array1_size = $#matrix_input + 1;
+   $array2_size = $#{$matrix_input[0]} + 1;
  
-   $min_size = $size_array1 < $size_array2 ? $size_array1 : $size_array2 ; # square matrix --> $min_size = $max_size and $size_array1 = $size_array2
-   $max_size = $size_array1 < $size_array2 ? $size_array2 : $size_array1 ;  
+   $min_size = $array1_size < $array2_size ? $array1_size : $array2_size ; # square matrix --> $min_size = $max_size and $array1_size = $array2_size
+   $max_size = $array1_size < $array2_size ? $array2_size : $array1_size ;
+   $original_max_size = $max_size;
    
    $maximize_total_benefit = $args{maximize_total_benefit};
    
@@ -43,26 +46,30 @@ sub auction { #                        => default values
    my %assignement_hash;  # assignement: a hash representing edges in the mapping, as in the Algorithm::Kuhn::Munkres.
    my @output_index;      # output_index: an array giving the number of the value assigned, as in the Algorithm::Munkres.
    
+   my @matrix;
+   my @matrix_index;
+   foreach ( @matrix_input ){ # copy the orginal matrix N x M
+      push @matrix, [ @$_ ];
+   }
+
    if ( $max_size <= 1 ){ # matrix_input 1 x 1
       $assignement_hash{0} = 0;
-      $output_index[0]     = 0;
-	  $optimal_benefit     = $matrix_input[0]->[0];
+      $output_index[0] = 0;
+	  $matrix_index[0] = 0;	
+	  $optimal_benefit = $matrix_input[0]->[0];
+   }
+
+   $need_transpose = 1 if ( $array1_size > $array2_size ); # will always be chosen N <= M
+   
+   if ( $need_transpose ){
+      my $transposed = transpose(\@matrix);
+      @matrix = @$transposed;
    }
    
-   my @matrix;
-   foreach ( @matrix_input ){ # copy the orginal matrix
-      push @matrix, [ @$_ ];
-   }   
-
    get_matrix_info( \@matrix, $args{verbose} );
-
-   for my $i ( 0 .. $max_size - 1 ) {
-   for my $j ( 0 .. $max_size - 1 ) {
-      next if ( defined $matrix_input[$i] and defined $matrix_input[$i]->[$j] );
-      # convert rectangular to square matrix by padding zeroes.			
-	  $matrix[$i]->[$j] = 0;
-   }}	
-
+   
+   delete_multiple_columns( \@matrix, $min_size, $args{verbose} ) if ( $min_size >= 2 );
+     
    # epsilon is the stepsize and auction algorithm terminates with a feasible assignment if the problem data are integer and epsilon < 1/min(N,M).
    # There is a trade-off between runtime and the chosen stepsize. Using the largest possible increment accelerates the algorithm.
    
@@ -73,7 +80,7 @@ sub auction { #                        => default values
    # The preceding observations suggest the idea of epsilon-scaling, which consists of applying the algorithm several times, 
    # starting with a large value of epsilon and successively reducing epsilon until it is less than some critical value.
    
-   while( $epsilon >= 1/(1+$max_size) and $max_size >= 2 ){
+   while( $epsilon >= 1/(1+$min_size) and $max_size >= 2 ){
    
 	  %assignned_object = ();
       %assignned_person = ();
@@ -83,7 +90,7 @@ sub auction { #                        => default values
          $iter_count++;
          auctionRound( \@matrix, $epsilon, $args{verbose} );
 		 
-		 if ( $args{verbose} >= 9 ){
+		 if ( $args{verbose} >= 10 ){
 		    @assignment = ();
 		    @prices = ();
             foreach my $per ( sort { $a <=> $b } keys %assignned_person){ push @assignment, $assignned_person{$per}; }
@@ -92,37 +99,142 @@ sub auction { #                        => default values
 		    print " *** \$iter_count = $iter_count ; \$assig_count = $assig_count ; \$epsilon = $epsilon ; \@assignment = (@assignment) ; \@prices = (@prices) \n\n";
          }		 
       }
+	  
 	  $epsilon = $epsilon * (1/4);
 	  
 	  if ( not $feasible_assignment_condition and $epsilon < 1/$min_size ){
-	     $epsilon = 1/(1+$max_size) ;
+	     $epsilon = 1/(1+$min_size) ;
 	  	 $feasible_assignment_condition = 1;
 	  }	  
    }
    $epsilon = 4 * $epsilon;
-   
-   foreach my $i ( sort { $a <=> $b } keys %assignned_person ){
+
+   my %seeN;
+   my %seeM;
+   foreach my $person ( sort { $a <=> $b } keys %assignned_person ){
 	  
-	  my $j = $assignned_person{$i};
+	  my $object = $assignned_person{$person};
+
+	  $matrix_index[$person] = $object;	  	 	  
+	  #print " \$need_transpose = $need_transpose ; \$matrix_index[$person] = $object ; \$index_i = $person ; \$index_j = $object --> $index_correlation{$object} ;";
+	  
+	  my $index_i = $need_transpose ? $index_correlation{$object} // $object : $person;
+      my $index_j = $need_transpose ? $person : $index_correlation{$object} // $object;
+
+      $output_index[$index_i] = $index_j; 
+      $seeN{$index_i}++;
+	  $seeM{$index_j}++;
+      #print " \$output_index[$index_i] = $index_j \n"; 	  
+	  
+	  next unless ( defined $matrix_input[$index_i] and defined $matrix_input[$index_i]->[$index_j] );
+	  $assignement_hash{ $index_i } = $index_j;	  
+	  $optimal_benefit += $matrix_input[$index_i]->[$index_j];
+   }
+
+   for my $i ( 0 .. $original_max_size - 1 ) {
+   for my $j ( 0 .. $original_max_size - 1 ) {
+      next if ($seeN{$i} or $seeM{$j});  
       $output_index[$i] = $j;
-	  
-	  next unless ( defined $matrix_input[$i] and defined $matrix_input[$i]->[$j] );
-	  $assignement_hash{ $i } = $j;	  
-	  $optimal_benefit += $matrix_input[$i]->[$j];
-   }   
+	  $seeN{$i}++;
+	  $seeM{$j}++;
+	  last;
+   }}   
    
    if ( $args{verbose} >= 10 ){
       printf "\n\$optimal_benefit = $optimal_benefit ; \$iter_count = $iter_count ; \$epsilon = %.4g ; \@output_index = (@output_index) ; \@assignment = (@assignment) ; \@prices = (@prices) \n", $epsilon; 
    }   
-   print_screen_messages( \@matrix, \@matrix_input, \@output_index, $optimal_benefit, $args{verbose}, $epsilon ) ;
+   print_screen_messages( \@matrix, \@matrix_index, \@matrix_input, \@output_index, $optimal_benefit, $args{verbose}, $epsilon ) ;
        
    return ( $optimal_benefit, \%assignement_hash, \@output_index ) ;
 }
 
+sub transpose {
+   my $matrix_ref = shift;
+   my @transpose;
+
+   for my $i ( 0 .. $#{$matrix_ref} ) {
+      for my $j ( 0 .. $#{$matrix_ref->[$i]} ) {
+	     $transpose[$j]->[$i] = $matrix_ref->[$i]->[$j];
+      }
+   }   
+   return \@transpose;
+}
+
+sub delete_multiple_columns { # if the column elements do not change the final result
+   my ( $matrix_ref, $N, $verbose ) = @_;
+   my @matrix = @$matrix_ref;
+   my %lower_values;
+   my %intersection_columns;
+   my $number_of_columns_deleted = 0;	  
+   
+   for my $i ( 0 .. $#{$matrix_ref} ) {
+      for my $j ( 0 .. $#{$matrix_ref->[$i]} ) {
+	     $lower_values{ $i }{ $matrix_ref->[$i]->[$j] }{ $j }++;
+      }
+   } 
+
+   foreach my $index_i ( sort { $a <=> $b } keys %lower_values ){
+      my $num_higher_values = 0;
+      foreach my $matrix_value ( sort { $b <=> $a } keys %{$lower_values{$index_i}} ){
+	     foreach my $index_j ( sort { $b <=> $a } keys %{$lower_values{$index_i}{$matrix_value}} ){	     
+		    $intersection_columns{$index_j}++ if ( ++$num_higher_values > $N );
+			$number_of_columns_deleted++ if ( defined $intersection_columns{$index_j} and $intersection_columns{$index_j} >= $N );
+		 }
+	  }
+   }
+
+   if ( $verbose >= 7 ){
+      print "\n";
+      for my $i ( 0 .. $#{$matrix_ref} ) {
+         print " [";
+         for my $j ( 0 .. $#{$matrix_ref->[$i]} ) {
+            printf(" %2.0f", $matrix_ref->[$i]->[$j] );
+			if ( defined $intersection_columns{$j} and $intersection_columns{$j} == $N ){ print "**"; } else{ print "  "; }
+         }
+         print "]\n";
+      }
+	  print "\n";
+   }	  
+
+   my $idx = 0;  
+   for my $i ( 0 .. $#{$matrix_ref} ) {
+      for my $j ( 0 .. $#{$matrix_ref->[$i]} ) {
+	     undef ( $matrix_ref->[$i]->[$j] ) if ( defined $intersection_columns{$j} and $intersection_columns{$j} >= $N );
+		 if ( $i == 0 and defined $intersection_columns{$j} and $intersection_columns{$j} >= $N ){
+		    print " \$N = $N ; \$intersection_columns{$j} = $intersection_columns{$j} \n" if ( $verbose >= 7 );
+		 }
+		 if ( $i == 0 and ( not defined $intersection_columns{$j} or $intersection_columns{$j} < $N ) ){
+		    $index_correlation{$idx} = $j;
+		    print " \$N = $N ; \$index_correlation{$idx} = $index_correlation{$idx} \n" if ( $verbose >= 7 );
+			$idx++;
+		 }
+      }
+   }
+   
+   for my $i ( 0 .. $#{$matrix_ref} ) {
+      @{$matrix_ref->[$i]} = grep { defined($_) } @{$matrix_ref->[$i]};
+   }
+  
+   if ( $verbose >= 7 ){
+      print "\n";
+      for my $i ( 0 .. $#{$matrix_ref} ) {
+         print " [";
+         for my $j ( 0 .. $#{$matrix_ref->[$i]} ) {
+            printf(" %2.0f  ", $matrix_ref->[$i]->[$j] );
+         }
+         print "]\n";
+      }
+	  print "\n";
+   }
+   
+   $max_size = $max_size - $number_of_columns_deleted;
+}
+
 sub print_screen_messages {
 
-   my ( $matrix_ref, $matrix_input_ref, $output_index_ref, $optimal_benefit, $verbose, $epsilon ) = @_;
+   my ( $matrix_ref, $matrix_index_ref, $matrix_input_ref, $output_index_ref, $optimal_benefit, $verbose, $epsilon ) = @_;
    my @matrix = @$matrix_ref;
+   my @matrix_index = @$matrix_index_ref;
    my @matrix_input = @$matrix_input_ref;
    my @output_index = @$output_index_ref;
    
@@ -130,9 +242,9 @@ sub print_screen_messages {
       
       print "\nObjective: ";
       printf( $maximize_total_benefit ? "to Maximize the total benefit\n" : "to Minimize the total benefit\n" );
-      printf(" Number of left nodes: %u\n",  $size_array1 );
-      printf(" Number of right nodes: %u\n", $size_array2 );
-      printf(" Number of edges: %u\n", $size_array1 * $size_array2 ); 
+      printf(" Number of left nodes: %u\n",  $array1_size );
+      printf(" Number of right nodes: %u\n", $array2_size );
+      printf(" Number of edges: %u\n", $array1_size * $array2_size ); 
 	  
 	  print "\nSolution:\n";	  
 	  printf(" Optimal assignment: sum of values = %.${decimals}f \n", $optimal_benefit );	  
@@ -153,9 +265,12 @@ sub print_screen_messages {
 	  
       print "\@output_index values  = [";
 	  
-      for my $i ( 0 .. $#matrix ){
+      for my $i ( 0 .. $#output_index ){
          my $j = $output_index[$i];
-         my $weight = ( defined $matrix_input[$i] and defined $matrix_input[$i]->[$j] ) ? sprintf( "%${matrix_spaces}.${decimals}f ", $matrix_input[$i]->[$j] ) : ' ' x ($matrix_spaces+1) ;
+		 last if not defined $j;
+		 my $weight;
+		    $weight = ( defined $matrix_input[$i] and defined $matrix_input[$i]->[$j] ) ? sprintf( "%${matrix_spaces}.${decimals}f ", $matrix_input[$i]->[$j] ) : ' ' x ($matrix_spaces+1) ;	 
+		 
          print $weight;
       }
       print "]\n\n";
@@ -163,40 +278,40 @@ sub print_screen_messages {
    
    if ( $verbose >= 2 ){
    
-      my $index_length = length($max_size);   
+      my $index_length = length($original_max_size);   
 
 	  if ( $verbose >= 4 ){
-      print " square matrix $max_size x $max_size with padding zeroes:\n";	  
+      printf " matrix %d x %d with modifications:\n", $#matrix + 1, $#{$matrix[0]} + 1;	  
       for my $i ( 0 .. $#matrix ) {
          print " [";
          for my $j ( 0 .. $#{$matrix[$i]} ) {
             printf(" %${matrix_spaces}.${decimals}f", $matrix[$i]->[$j] );
-            if ( $j == $output_index[$i] ){ print "**"; } else{ print "  "; }
+            if ( $j == $matrix_index[$i] ){ print "**"; } else{ print "  "; }
          }
          print "]\n";
       }
 	  print "\n";
 	  }
 	  
-	  print " original matrix $size_array1 x $size_array2 with solution:\n";
+	  print " original matrix $array1_size x $array2_size with solution:\n";
 
       for my $i ( 0 .. $#matrix_input ) {
          print " [";
          for my $j ( 0 .. $#{$matrix_input[$i]} ) {
-            printf(" %${matrix_spaces}.${decimals}f", $matrix_input[$i]->[$j] );
-            if ( $j == $output_index[$i] ){ print "**"; } else{ print "  "; }
+            printf(" %${matrix_spaces}.${decimals}f", $matrix_input[$i]->[$j] );			
+			if ( $j == $output_index[$i] ){ print "**"; } else{ print "  "; }		
          }
          print "]\n";
       }
 	  
       my %orderly_solution;
-      for my $i ( 0 .. $#matrix ){
+      for my $i ( 0 .. $original_max_size - 1 ){
          my $j = $output_index[$i];
          my $weight = $max_matrix_value;
          $weight = $matrix_input[$i]->[$j] if ( defined $matrix_input[$i] and defined $matrix_input[$i]->[$j] ); # condition for valid solution
-   
+         
          $orderly_solution{ $weight } { $i } { 'index_array1' } = $i;
-         $orderly_solution{ $weight } { $i } { 'index_array2' } = $j; 
+         $orderly_solution{ $weight } { $i } { 'index_array2' } = $j;		 
       }
 
       print "\n Pairs (in ascending order of weight function values):\n"; 
@@ -237,18 +352,21 @@ sub get_matrix_info {
    $decimals = length(($max_matrix_value =~ /[,.](\d+)/)[0]); # counting the number of digits after the decimal point
    $decimals = 0 unless ( defined $decimals );                # for integers $decimals = 0
    
-   my $gap = $max_matrix_value - $min_matrix_value;           # $gap >= 0
-      $gap = 1 if ($gap == 0);
+   my $range = $max_matrix_value - $min_matrix_value;           # $range >= 0
+      $range = 1 if ($range == 0);
    
-   if ( $verbose >= 5 ){
-      print "\n min_matrix_value = $min_matrix_value ; max_matrix_value = $max_matrix_value ; gap = $gap ; matrix_spaces = $matrix_spaces ; decimals = $decimals \n";
+   if ( $verbose >= 6 ){
+      print "\n min_matrix_value = $min_matrix_value ; max_matrix_value = $max_matrix_value ; range = $range ; matrix_spaces = $matrix_spaces ; decimals = $decimals \n";
    }
    
    if ( $maximize_total_benefit ){
 
       for my $i ( 0 .. $#matrix ) {
       for my $j ( 0 .. $#{$matrix[$i]} ) {
-	     $matrix[$i]->[$j] = 99 * ( $matrix[$i]->[$j] - $min_matrix_value ) / $gap; # new scale: Min = 0 <---> Max = 99
+	     
+		 #$matrix[$i]->[$j] = $matrix[$i]->[$j] - $min_matrix_value ;
+		 
+	     $matrix[$i]->[$j] = 99 * ( $matrix[$i]->[$j] - $min_matrix_value ) / $range; # new scale: Min = 0 <---> Max = 99
       }}   
  	  
    } else {
@@ -256,9 +374,9 @@ sub get_matrix_info {
       for my $i ( 0 .. $#matrix ) {
       for my $j ( 0 .. $#{$matrix[$i]} ) {   
 	     
-		 #$matrix[$i]->[$j] = ( $max_matrix_value - $matrix[$i]->[$j] );
+		 #$matrix[$i]->[$j] = $max_matrix_value - $matrix[$i]->[$j] ;
 		 
-		 $matrix[$i]->[$j] = 99 * ( $max_matrix_value - $matrix[$i]->[$j] ) / $gap;
+		 $matrix[$i]->[$j] = 99 * ( $max_matrix_value - $matrix[$i]->[$j] ) / $range;
       }}	  
    }
 }
@@ -271,16 +389,21 @@ sub auctionRound {
    if ( $verbose >= 10 ){
       print "\n Start: Matrix Size N x M: $max_size x $max_size ; Num Iterations: $iter_count ; epsilon: $epsilon \n";
 	  foreach my $person ( sort { $a <=> $b } keys %assignned_person ){
-	     print " \$assignned_person{$person} --> object $assignned_person{$person} \n";
+	     my $object = $assignned_person{$person};
+	     printf " \$assignned_person{%2s} --> object %2s --> \$price_object{%2s} = $price_object{$object} \n", $person, $object, $object;
 	  }
 	  foreach my $object ( sort { $a <=> $b } keys %price_object ){
-	     print " \$price_object{$object} = $price_object{$object} \n";
+	     printf " \$price_object{%2s} = $price_object{$object} \n", $object;
 	  }
 	  print "\n";
    }
+   
+   my $seen_ghost;
 
-	for my $person ( 0 .. $#matrix )
-	{   	    
+	for my $person ( 0 .. $max_size - 1 )
+	{		
+		last if $seen_ghost; # don't need to fill the matrix with zeros, that is, don't need to convert rectangular N x M to square matrix by padding zeroes.
+		
 		if ( not defined $assignned_person{$person} )
 		{		
 			##     ------> j          object 0   object 1   object 2   object 3 ...  object (M - 1)
@@ -294,10 +417,14 @@ sub auctionRound {
 
 			my ( $optValForI, $secOptValForI, $optObjForI, $secOptObjForI );
 			
-			for my $object ( 0 .. $#matrix )
+			for my $object ( 0 ..  $max_size - 1 )
 			{				
+				$seen_ghost++ if ( not defined $matrix[$person] );
+				
+				my $matrix_value = $seen_ghost ? 0 : $matrix[$person]->[$object] ;				
+				
 				$price_object{$object} = $inicial_price unless ( defined $price_object{$object} );				
-				my $curVal = $matrix[$person]->[$object] - $price_object{$object};               					
+				my $curVal = $matrix_value - $price_object{$object};				
 				
 				if ( (not defined $optValForI) || ($curVal > $optValForI) )
 				{
@@ -313,9 +440,9 @@ sub auctionRound {
 				}
 				
 				if ( $verbose >= 10 ){
-			        printf " personI = %2s ; objectJ = %2s ; \$size_array1 = %2s ; \$size_array2 = %2s ; \$curVal %8.4f = \$matrix[%2s]->[%2s] %8.4f - \$price_object{%2s} %8.4f ;", $person, $object, $size_array1, $size_array2, $curVal, $person, $object, $matrix[$person]->[$object], $object, $price_object{$object};				
-				    if ( defined $optValForI    ){ printf " \$optValForI = %8.4f ", $optValForI; }
-				    if ( defined $secOptValForI ){ printf " \$secOptValForI = %8.4f \n", $secOptValForI; } else { print "\n"; }
+			        printf " personI = %2s ; objectJ = %2s ; N = %2s ; M = %2s ; \$curVal %6.2f = \$matrix[%2s]->[%2s] %2.0f - \$price_object{%2s} %6.2f ;", $person, $object, $array1_size, $array2_size, $curVal, $person, $object, $matrix_value, $object, $price_object{$object};				
+				    if ( defined $optValForI    ){ printf " \$optValForI = %6.2f ", $optValForI; }
+				    if ( defined $secOptValForI ){ printf " \$secOptValForI = %6.2f \n", $secOptValForI; } else { print "\n"; }
                 }
 			}
 
@@ -347,7 +474,6 @@ sub auctionRound {
 			 print " ***--> PersonI $person was assigned objectJ $object. Before that, remove the objectJ $object from personI $other_p  --> delete \$assignned_person{$other_p} \n";
 	      }
 		  delete $assignned_person{$other_p};
-	      delete $assignned_object{$object};
 	   }
 	   
 	   $assignned_person{$person} = $object;
@@ -356,18 +482,19 @@ sub auctionRound {
 	   $price_object{$object} += $bid;
 	   
 	   if ( $verbose >= 10 ){
-	      printf " --> Assigning to personI = $person the objectJ = $object with highestBidForJ = %8.4f and update the price vector ; \$assignned_person{$person} = $assignned_person{$person} ; \$price_object{$object} = $price_object{$object} \n", $bid;	
+	      printf " --> Assigning to personI = $person the objectJ = $object with highestBidForJ = %6.2f and update the price vector ; \$assignned_person{$person} = $assignned_person{$person} ; \$price_object{$object} = $price_object{$object} \n", $bid;	
        }
-    }}}	
+    }}}
 	
    # Prints the %assignned_person and %price_object
    if ( $verbose >= 10 ){
       print "\n Final: Matrix Size N x M: $max_size x $max_size ; Num Iterations: $iter_count ; epsilon: $epsilon \n";
 	  foreach my $person ( sort { $a <=> $b } keys %assignned_person ){
-	     print " \$assignned_person{$person} --> object $assignned_person{$person} \n";
+	     my $object = $assignned_person{$person};
+	     printf " \$assignned_person{%2s} --> object %2s --> \$price_object{%2s} = $price_object{$object} \n", $person, $object, $object;
 	  }
 	  foreach my $object ( sort { $a <=> $b } keys %price_object ){
-	     print " \$price_object{$object} = $price_object{$object} \n";
+	     printf " \$price_object{%2s} = %17.14f \n", $object, $price_object{$object};
 	  }
    }
    
@@ -381,23 +508,19 @@ __END__
 
     Algorithm::Bertsekas - auction algorithm for the assignment problem.
 	
-	This is a perl implementation for the auction algorithm for the symmetric allocation problem.
-	
-	"Both, the auction algorithm and the Kuhn-Munkres algorithm have worst-case time complexity 
-	of (roughly) O(N^3). However, the average-case time complexity of the auction algorithm is 
-	much better. Thus, in practice, with respect to running time, the auction algorithm outperforms 
-	the Kuhn-Munkres (or Hungarian) algorithm significantly."
+	This is a perl implementation for the auction algorithm for the assymmetric (N<=M) allocation problem.
 
 =head1 DESCRIPTION
  
  The assignment problem in the general form can be stated as follows:
 
- "Given N jobs, M tasks and the effectiveness of each job for each task, the problem is to assign each job 
- to one and only one task in such a way that the measure of effectiveness is optimised (Maximised or Minimised)."
+ "Given N jobs (or persons), M tasks (or objects) and the effectiveness of each job for each task, 
+ the problem is to assign each job to one and only one task in such a way that the measure of 
+ effectiveness is optimised (Maximised or Minimised)."
  
- "Each assignment problem has associated with a table or matrix. Generally, the rows contain the jobs or people 
- we wish to assign, and the columns comprise the tasks or things we want them assigned to. The numbers in the 
- table are the costs associated with each particular assignment."
+ "Each assignment problem has associated with a table or matrix. Generally, the rows contain the 
+ jobs or people we wish to assign, and the columns comprise the tasks or objects we want them 
+ assigned to. The numbers in the table are the costs associated with each particular assignment."
  
  One application is to find the (nearest/more distant) neighbors. 
  The distance between neighbors can be represented by a matrix or a weight function, for example:
@@ -409,8 +532,8 @@ __END__
 
  use Algorithm::Bertsekas qw(auction);
 
- my @array1 = ( 64.68, 47.56,  7.36, 80.90, 96.71, 50.10, 44.16 );
- my @array2 = (  3.91, 88.77, 45.56, 79.28 );
+ my @array1 = ( 19, 74, 58,  1 )
+ my @array2 = ( 94, 55, 90, 45, 56, 95, 90 );
  
  my $min = $#array1 < $#array2 ? $#array1 : $#array2;
  my $max = $#array1 < $#array2 ? $#array2 : $#array1;
@@ -425,58 +548,50 @@ __END__
    push @input_matrix, \@weight_function;
  } 
  
-         3.91 88.77 45.56 79.28
+      94 55 90 45 56 95 90
 
- 64.68 [ 60.77 24.09 19.12 14.60 ]
- 47.56 [ 43.65 41.21  2.00 31.72 ]
-  7.36 [  3.45 81.41 38.20 71.92 ]
- 80.90 [ 76.99  7.87 35.34  1.62 ]
- 96.71 [ 92.80  7.94 51.15 17.43 ]
- 50.10 [ 46.19 38.67  4.54 29.18 ]
- 44.16 [ 40.25 44.61  1.40 35.12 ]
+ 19 [ 75 36 71 26 37 76 71 ]
+ 74 [ 20 19 16 29 18 21 16 ]
+ 58 [ 36  3 32 13  2 37 32 ]
+  1 [ 93 54 89 44 55 94 89 ]
 
  @input_matrix = (
- [ 60.77, 24.09, 19.12, 14.60 ],
- [ 43.65, 41.21,  2.00, 31.72 ],
- [  3.45, 81.41, 38.20, 71.92 ],
- [ 76.99,  7.87, 35.34,  1.62 ],
- [ 92.80,  7.94, 51.15, 17.43 ],
- [ 46.19, 38.67,  4.54, 29.18 ],
- [ 40.25, 44.61,  1.40, 35.12 ]
+ [ 75, 36, 71, 26, 37, 76, 71 ],
+ [ 20, 19, 16, 29, 18, 21, 16 ],
+ [ 36,  3, 32, 13,  2, 37, 32 ],
+ [ 93, 54, 89, 44, 55, 94, 89 ]
  );
+ 
+ my ( $optimal, $assignement_ref, $output_index_ref ) = auction( matrix_ref => \@input_matrix, maximize_total_benefit => 1, verbose => 3 );
 
- my ( $optimal, $assignement_ref, $output_index_ref ) = auction( matrix_ref => \@input_matrix, maximize_total_benefit => 0, verbose => 2 );
-
-Objective: to Minimize the total benefit
- Number of left nodes: 7
- Number of right nodes: 4
+Objective --> to Maximize the total benefit
+ Number of left nodes: 4
+ Number of right nodes: 7
  Number of edges: 28
 
-Solution:
- Optimal assignment: sum of values = 14.41
+Solution
+ Optimal assignment, sum of values = 230
  Feasible assignment condition: stepsize = 0.2 < 1/4 = 0.25
+ Number of iterations: 18
 
-Maximum index size    = [    0     1     2     3     4     5     6 ]
-@output_index indexes = [    6     5     0     3     1     4     2 ]
-@output_index values  = [             3.45  1.62  7.94        1.40 ]
+Maximum index size    = [ 0  1  2  3  4  5  6 ]
+@output_index indexes = [ 5  3  6  0  2  1  4 ]
+@output_index values  = [76 29 32 93          ]
 
- original matrix 7 x 4 with solution:
- [ 60.77   24.09   19.12   14.60  ]
- [ 43.65   41.21    2.00   31.72  ]
- [  3.45** 81.41   38.20   71.92  ]
- [ 76.99    7.87   35.34    1.62**]
- [ 92.80    7.94** 51.15   17.43  ]
- [ 46.19   38.67    4.54   29.18  ]
- [ 40.25   44.61    1.40** 35.12  ]
+ original matrix 4 x 7 with solution:
+ [ 75   36   71   26   37   76** 71  ]
+ [ 20   19   16   29** 18   21   16  ]
+ [ 36    3   32   13    2   37   32**]
+ [ 93** 54   89   44   55   94   89  ]
 
  Pairs (in ascending order of weight function values):
-   indexes ( 6, 2 ), weight value =  1.40 ; sum of values =  1.40
-   indexes ( 3, 3 ), weight value =  1.62 ; sum of values =  3.02
-   indexes ( 2, 0 ), weight value =  3.45 ; sum of values =  6.47
-   indexes ( 4, 1 ), weight value =  7.94 ; sum of values = 14.41
-   indexes ( 0, 6 ), weight value =       ; sum of values = 14.41
-   indexes ( 1, 5 ), weight value =       ; sum of values = 14.41
-   indexes ( 5, 4 ), weight value =       ; sum of values = 14.41
+   indexes ( 1, 3 ), weight value = 29 ; sum of values = 29
+   indexes ( 2, 6 ), weight value = 32 ; sum of values = 61
+   indexes ( 0, 5 ), weight value = 76 ; sum of values = 137
+   indexes ( 3, 0 ), weight value = 93 ; sum of values = 230
+   indexes ( 4, 2 ), weight value =    ; sum of values = 230
+   indexes ( 5, 1 ), weight value =    ; sum of values = 230
+   indexes ( 6, 4 ), weight value =    ; sum of values = 230
 
 
  Common use of the solution:
@@ -491,19 +606,6 @@ Maximum index size    = [    0     1     2     3     4     5     6 ]
    my $j = $output_index_ref->[$i];
    ...
  }
- 
- assignement hash --> $i =   2 e $value1 =  7.36; $j =   0 e $value2 =  3.91 ; difference =  3.45 ; sum =  3.45
- assignement hash --> $i =   3 e $value1 = 80.90; $j =   3 e $value2 = 79.28 ; difference =  1.62 ; sum =  5.07
- assignement hash --> $i =   4 e $value1 = 96.71; $j =   1 e $value2 = 88.77 ; difference =  7.94 ; sum = 13.01
- assignement hash --> $i =   6 e $value1 = 44.16; $j =   2 e $value2 = 45.56 ; difference =  1.40 ; sum = 14.41
-
- output_index array --> $i =   0 e $value1 = 64.68; $j =   6 e $value2 =       ; difference =       ; sum =  0.00
- output_index array --> $i =   1 e $value1 = 47.56; $j =   5 e $value2 =       ; difference =       ; sum =  0.00
- output_index array --> $i =   2 e $value1 =  7.36; $j =   0 e $value2 =  3.91 ; difference =  3.45 ; sum =  3.45
- output_index array --> $i =   3 e $value1 = 80.90; $j =   3 e $value2 = 79.28 ; difference =  1.62 ; sum =  5.07
- output_index array --> $i =   4 e $value1 = 96.71; $j =   1 e $value2 = 88.77 ; difference =  7.94 ; sum = 13.01
- output_index array --> $i =   5 e $value1 = 50.10; $j =   4 e $value2 =       ; difference =       ; sum = 13.01
- output_index array --> $i =   6 e $value1 = 44.16; $j =   2 e $value2 = 45.56 ; difference =  1.40 ; sum = 14.41
 
  
 =head1 OPTIONS
